/*
 * LatexTabellenLogger.cpp
 *
 *  Created on: 28.01.2009
 *      Author: knoeohcd
 */

#include "LatexTabellenLogger.h"
#include <sstream>
#include <cmath>

LatexTabellenLogger::LatexTabellenLogger(const string &Dateiname,unsigned int Spalten,unsigned int ExpSchr,const char *localestr)  throw(std::runtime_error) : spalten(Spalten), akt_sp(0), header_geschr(false)
{
	maxv = pow(10,ExpSchr);
	minv = pow(10,-1.0*ExpSchr);
	try{
		ausgabe.open(Dateiname.c_str());
		ausgabe << "\\begin{table}[h]" << endl
		<< " \\centering" << endl
		<< " \\begin{tabular}{|";
		for(unsigned int k=0;k<spalten;k++) { ausgabe << "c|"; }
		ausgabe << "}\\hline" << endl << " ";
		ausgabe.precision(ExpSchr);
		ausgabe.imbue(locale(localestr));
	}
	catch(const ios::failure &e)
	{
		throw runtime_error(string("LatexTabellenLogger::LatexTabellenLogger: E/A-Fehler ")+string(e.what()).c_str());
	}
	catch(...)
	{
		throw runtime_error("LatexTabellenLogger::LatexTabellenLogger: unbekannter Ausnahmefehler.");
	}
}


LatexTabellenLogger::~LatexTabellenLogger()
{
	ausgabe << "\\hline" << endl
		<< " \\end{tabular}" << endl;
	if(!caption.empty()) ausgabe << " \\caption{" << caption << "}" << endl;
	if(!label.empty()) ausgabe << " \\label{" << label << "}" << endl;
	ausgabe << "\\end{table}" << endl;
	ausgabe.close();
}

LatexTabellenLogger &LatexTabellenLogger::operator<<(string wert)
{
	ausgabe << wert;
	if ((akt_sp = (akt_sp + 1) % spalten) == 0)
	{
		// neue Zeile
		ausgabe << "\\\\";
		if (!header_geschr)
		{
			header_geschr = true;
			ausgabe << " \\hline";
		}
		ausgabe << endl << " ";
	} else
	{
		ausgabe << " & ";
	}
	return *this;
}

LatexTabellenLogger &LatexTabellenLogger::operator<<(long double wert)
{
	ostringstream ausgabestr;
	ausgabestr << "$";
	long double betr=abs(wert);
	if(betr != 0.0 && (betr > maxv || betr < minv))
	{
		// Exponentialausgabe
		// bestimme den Exponenten
		long int exp = floor(log10(betr));
		wert/=pow(10,exp);
		ausgabestr << wert << "\\cdot 10^{" << exp << "} ";
	}
	else
	{
		// normale Ausgabe
		ausgabestr << wert;
	}
	ausgabestr << "$";
	return operator<<(ausgabestr.str());
}



LatexTabellenLogger &LatexTabellenLogger::operator<<(int wert)
{
	ostringstream ausgabestr;
	ausgabestr << "$";
	long double betr=abs((long double) wert);
	if(betr != 0.0 && (betr > maxv || betr < minv))
	{
		// Exponentialausgabe
		// bestimme den Exponenten
		long int exp = floor(log10(betr));
		wert/=pow(10,exp);
		ausgabestr << wert << "\\cdot 10^{" << exp << "} ";
	}
	else
	{
		// normale Ausgabe
		ausgabestr << wert;
	}
	ausgabestr << "$";
	return operator<<(ausgabestr.str());
}

LatexTabellenLogger &LatexTabellenLogger::operator<<(unsigned int wert)
{
	ostringstream ausgabestr;
	ausgabestr << "$";
	if(wert != 0.0 && wert > maxv)
	{
		// Exponentialausgabe
		// bestimme den Exponenten
		long int exp = floor(log10(wert));
		wert/=pow(10,exp);
		ausgabestr << wert << "\\cdot 10^{" << exp << "} ";
	}
	else
	{
		// normale Ausgabe
		ausgabestr << wert;
	}
	ausgabestr << "$";
	return operator<<(ausgabestr.str());
}
